# frozen_string_literal: true

# An AcademicCard provides a topical way to filter and search for an academic
# program.
class AcademicCard
  include Mongoid::Document
  include Mongoid::Timestamps
  include Elasticsearchable

  index_name 'biola_academic_cards'

  field :title, type: String
  field :description, type: String

  belongs_to :school
  has_and_belongs_to_many :academic_programs

  validates :title, :description, presence: true

  mapping do
    indexes :title, type: 'multi_field' do
      indexes :raw, index: 'not_analyzed'
      indexes :snow, analyzer: 'snowball'
      indexes :edge, analyzer: 'edge_ngram_analyzer'
    end
    indexes :description, type: 'string', analyzer: 'standard'
    indexes :school, type: 'object' do
      indexes :title, type: 'multi_field' do
        indexes :snow, analyzer: 'snowball'
        indexes :edge, analyzer: 'edge_ngram_analyzer'
      end
      indexes :slug, index: 'not_analyzed'
    end
    indexes :academic_programs, type: 'nested' do
      indexes :title, type: 'multi_field' do
        indexes :snow, analyzer: 'snowball'
        indexes :edge, analyzer: 'edge_ngram_analyzer'
      end
      indexes :academic_program_formats, type: 'object' do
        indexes :title
        indexes :slug, index: 'not_analyzed'
      end
      indexes :academic_program_types, type: 'object' do
        indexes :title
        indexes :slug, index: 'not_analyzed'
      end
    end
  end

  def as_indexed_json(*)
    as_json(
      only: %i[title description],
      include: {
        school: { only: %i[title slug] },
        academic_programs: {
          only: %i[page_url level],
          methods: :title,
          include: {
            academic_program_formats: { only: %i[title slug] },
            academic_program_types: { only: %i[title slug] },
            academic_subjects: { only: %i[title slug] }
          }
        }
      }
    )
  end
end
