# AcademicProgram:
class AcademicProgram
  include Mongoid::Document
  include Mongoid::Slug
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include RelatesObjectTags
  include HasPresentationData
  include Elasticsearchable
  include StringAccessorForArray
  include ActsAsWebPage
  include HasPages

  COMPLETION_AWARDS = [
    { level: :bachelor, degrees: ['B.A.', 'B.B.A.', 'B.S.', 'B.M.', 'B.F.A.', 'Certificate', 'Credential', 'Permit', 'Minor'] },
    { level: :graduate, degrees: ['Certificate', 'Credential', 'Permit', 'M.A.', 'M.A.C.E.', 'M.A.C.M.L.', 'M.A.Ed.', 'M.A.S.F.S.C.', 'M.A.T.', 'M.B.A.', 'M.Div.', 'MPAcc', 'M.S.', 'Th.M.'] },
    { level: :doctoral, degrees: ['Ph.D.', 'Psy.D.', 'Ed.D.', 'D.Min.', 'D.Miss.'] },
    { level: :non_degree, degrees: [] }
  ].freeze
  DEGREES = COMPLETION_AWARDS.collect { |award| award[:degrees] }.flatten.uniq
  LEVELS = COMPLETION_AWARDS.collect {|award| award[:level]}.uniq
  # only use these attachment types in the attachment metadata field
  ATTACHMENT_TYPES = %w(plo-assessment-report curriculum-map assessment-plan assessment-schedule executive-summary-review)

  ATTACHMENT_METADATA_FIELDS = [
    FormFieldDefinition.new(name: 'type', label: 'Type', type: 'select', options_for_select: ATTACHMENT_TYPES.map{|t| [t,t]}),
    FormFieldDefinition.new(name: 'year', label: 'Year', type: 'text_field', help_text: 'Examples: 2014 or Fall 2014'),
    FormFieldDefinition.new(name: 'course_key', label: 'Course Key', type: 'text_field', help_text: 'Example: BBST 251')
  ]

  CATEGORIES = ['Degree Completion', 'Online', 'Off-Campus Program'].sort

  mount_uploader :banner, GenericPhotoUploader

  # This is for relating faculty members.
  embeds_many :memberships, as: :unitable

  # APPS-1097: Forces the related AcademicCards to reindex when the
  # AcademicProgram is updated. We'll change this in 2.0.
  has_and_belongs_to_many :academic_cards
  after_save -> { academic_cards.each(&:save) }

  belongs_to :school
  has_and_belongs_to_many :academic_subjects
  has_and_belongs_to_many :academic_program_formats
  has_and_belongs_to_many :academic_program_types
  embeds_many :learning_outcomes
  embeds_many :catalog_entries, as: :cataloged
  embeds_many :requirement_sections
  has_many :attachments, as: :attachable
  has_many :concentrations, dependent: :delete
  has_many :degree_requirements
  before_validation :set_level, unless: :level?
  before_validation :force_nil_degree, if: "degree == ''"

  field :catalog_url, type: String
  # 'Education, M.A.' or 'Master of Arts, Talbot'
  field :full_name, type: String
  # Optional: Overwrites title in informal lists like Undergrad Major page
  field :nickname, type: String
  field :discipline, type: String # Example: Biblical and Theological Studies
  field :level, type: Symbol # :undergraduate
  field :program_formats, type: Array, default: [] # Standard, Online, Hybrid
  field :program_types, type: Array, default: []
  field :mission, type: String
  field :degree, type: String # Example: B.A. or Credential
  field :outcomes_intro, type: String
  field :plo_revision_date, type: Date
  field :next_cpr_date, type: String
  # Example: 'online' of 'degree completion'; Academic Publisher will control
  # the set of values for this field
  field :categories, type: Array, default: []
  field :external_url, type: String # For storing url to external website
  field :biola_link_id, type: String # For BiolaLink Embedded codes
  field :career_titles, type: Array # Acts as a filter for related careers
  # For relating other academic programs, these are not related object tags
  field :tags, type: Array
  string_accessor_for_array :career_titles
  string_accessor_for_array :tags
  slug :full_name
  # Program at a glance
  field :program_credits, type: String
  field :general_education_credits, type: String
  field :major_integrated_bible_theology_credits, type: String
  field :bible_theology_credits, type: String
  field :major_concentration_credits, type: String
  field :standard_completion_time, type: String
  field :accreditation, type: String
  field :support_electives, type: String
  field :specializations, type: String
  # Published to
  field :published_to_production, type: Boolean, default: false
  field :published_to_outcomes, type: Boolean, default: false
  # For storing url to a program's application if there is one
  field :application_url, type: String
  field :request_information_url, type: String
  field :schedule_visit_url, type: String
  # students applying for this major also must have one of the
  # available concentrations
  field :requires_concentration, type: Boolean, default: false

  validates :full_name, presence: true, uniqueness: true
  validates :slug, uniqueness: true, allow_blank: true
  validates :external_url, absolute_uri: true, allow_blank: true
  validates :discipline, presence: true
  validates :level, inclusion: { in: AcademicProgram::LEVELS, allow_nil: false }

  validates :degree, inclusion: {
    in: AcademicProgram::DEGREES,
    allow_nil: true
  }
  validates :application_url, absolute_uri: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true,
                track_destroy: true

  alias to_s full_name
  alias title full_name

  scope :with_masters_degree, (lambda do
    where(level: :graduate).not
                           .in(degree: %w[Certificate Credential Permit Minor])
  end)

  scope :minors, -> { where(level: :bachelor, degree: 'Minor') }
  scope :without_minors, -> { where(:degree.ne => 'Minor') }

  scope :with_bachelors_degree, (lambda do
    where(level: :bachelor).not
                           .in(degree: %w[Certificate Credential Permit Minor])
  end)

  scope :with_non_degree_programs, (lambda do
    where(level: :non_degree)
  end)

  scope :with_doctoral_degree, (lambda do
    where(level: :doctoral).not
                           .in(degree: %w[Certificate Credential Permit Minor])
  end)

  scope :by_level, ->(level) { where(level: level.to_sym) }
  scope :by_degree, ->(degree) { where(degree: degree.titleize) }
  scope :by_school, ->(school_id) { where(school_id: school_id) }
  scope :by_category, ->(category) { where(categories: category) }
  scope :by_tag, ->(tag) { where(tags: tag) }
  scope :degree_completion, -> { by_category('Degree Completion') }
  scope :online, -> { by_category('Online') }

  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({ full_name: q }, { discipline: q }, { degree: q })
      end
    rescue ::RegexpError
      AcademicProgram.none
    end
  end)

  # ensure nil and not empty string
  def force_nil_degree
    self.degree = degree.presence
  end

  # method for finding attachments with 'plo-assessment-report' metadata type
  def plo_reports
    attachments.where('metadata.type' => 'plo-assessment-report').to_a
  end

  # method for finding attachments with 'curriculum-map' metadata type
  def curriculum_maps
    attachments.where('metadata.type' => 'curriculum-map').to_a
  end

  # method for finding attachments with 'assessment-plan' metadata type
  def assessment_plans
    attachments.where('metadata.type' => 'assessment-plan').to_a
  end

  # method for finding attachments with 'assessment-schedule' metadata type
  def assessment_schedules
    attachments.where('metadata.type' => 'assessment-schedule').to_a
  end

  # method for finding attachments with 'executive-summary-review' metadata type
  def pr_executive_summaries
    attachments.where('metadata.type' => 'executive-summary-review').to_a
  end

  # Find all other programs with similar tags
  def related_programs
    AcademicProgram.none unless tags.present?
    AcademicProgram.where(tags: { '$in' => Array(tags) })
                   .where(:id.ne => id.to_s)
  end

  def as_indexed_json(*)
    {
      title: title,
      slug: slug,
      level: level,
      degree: degree,
      discipline: discipline
    }
  end

  mapping do
    # TODO: Look into using multi field type for this.
    #  http://www.elasticsearch.org/guide/en/elasticsearch/reference/0.90/mapping-multi-field-type.html#mapping-multi-field-type
    indexes 'title', type: 'string', fields: {
      raw:  { type: 'string', index: 'not_analyzed' },
      edge: { type: 'string', analyzer: 'edge_ngram_analyzer' }
    }
  end

  def dont_index?
    !published_to_production
  end

  private

  def set_level
    return unless degree?
    COMPLETION_AWARDS.each do |grouping|
      # last match wins
      self.level = grouping[:level] if grouping[:degrees].include?(degree)
    end
  end
end
