# AcademicSubject:
class AcademicSubject
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include RelatesObjectTags
  include HasPresentationData
  include ActsAsWebPage
  include HasPages

  # You have to do this both before validate and before save because when you
  # run .update_attribute(:slug, ...) it won't run the before validation
  # callback. But when you initially create it won't run the before_save
  # callback before the validation runs, so it will not validate since the
  # path is required.
  before_validation :generate_path
  before_save :generate_path
  after_save :regenerate_child_paths

  has_and_belongs_to_many :academic_programs
  has_and_belongs_to_many :schools
  has_and_belongs_to_many :departments

  # This is for defining a hierarchical relationship between subjects.
  # Should mimic path.
  belongs_to :parent_subject, class_name: 'AcademicSubject',
                              inverse_of: :child_subject
  has_many :child_subjects, class_name: 'AcademicSubject',
                            inverse_of: :parent_subject

  # This is for relating any two Academic Topics
  has_and_belongs_to_many :related_subjects, class_name: 'AcademicSubject',
                                             inverse_of: :related_subjects

  # This is for relating faculty members.
  embeds_many :memberships, as: :unitable

  field :title, type: String
  field :slug, type: String
  field :path, type: String
  # this is used by academics front end to specify which template
  # the subject should use.
  field :template_name, type: String

  # Published to
  field :published_to_production, type: Boolean, default: false
  field :published_to_outcomes, type: Boolean, default: false

  validates :title, uniqueness: true, presence: true
  validates :slug, uniqueness: true, presence: true
  validates :path, uniqueness: true, presence: true

  track_history on: :fields, track_create: true, track_update: true,
                track_destroy: true

  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of(title: q)
      end
    rescue ::RegexpError
      AcademicSubject.none
    end
  end)

  alias to_s title

  # Returns an Array of parents from furthest to closest.
  #
  #   dad.parent_subject = grandpa
  #   me.parent_subject = dad
  #   me.parents # => [grandpa, dad]
  def parents
    if parent_subject.present?
      # Recursivly call this method. If parent doesn't have a parent,
      # it will return []
      temp_parents = parent_subject.parents

      # prevent against cyclic relations
      if temp_parents.include? parent_subject
        temp_parents
      else
        temp_parents + [parent_subject]
      end
    else
      []
    end
  end

  private

  # Generates the path of this object in relation to its parents.
  # This does not update children, or save.
  #
  # biology_subject.generate_path # =>
  #   "sciences/biological-physical-sciences/biology"
  def generate_path
    self.path = (parents + [self]).map(&:slug).join('/')
  end

  # Loops through child paths and saves them, which triggers a call to
  # `generate_path`. Also triggers a call to `regenerate_child_paths`
  # on each child.
  # Note: It only runs if the path was changed.
  def regenerate_child_paths
    child_subjects.each(&:save) if changed_attributes['path']
  end
end
