class ActivityLog
  include Mongoid::Document
  include Mongoid::Timestamps

  unless const_defined?(:ACTIONABLE_ACTIONS)
    ACTIONABLE_ACTIONS = [
      'update',
      'update_from_trogdir',
      'create_from_trogdir',
      'destroy_from_trogdir',
      'create',
      'destroy',
      'publish',
      'unpublish',
      'archive',
      'approve',
      'reject',
      'skip',
      'request_fact_check',
      'import_from_ws',
      'update_from_ws',
      'export_to_ws'
    ]
  end

  unless const_defined?(:NON_CRUD_ACTIONABLE_ACTIONS)
    NON_CRUD_ACTIONABLE_ACTIONS = [
      'approve',
      'reject',
      'skip',
      'request_fact_check'
    ]
  end

  unless const_defined?(:TROGDIR_ACTIONABLE_ACTIONS)
    TROGDIR_ACTIONABLE_ACTIONS = [
      'update_from_trogdir',
      'create_from_trogdir',
      'destroy_from_trogdir'
    ]
  end

  unless const_defined?(:WS_ACTIONABLE_ACTIONS)
    WS_ACTIONABLE_ACTIONS = [
      'import_from_ws',
      'update_from_ws',
      'export_to_ws'
    ]
  end

  field :associated_id,     type: String
  field :acting_user,       type: String
  field :acting_user_email, type: String
  field :reviewer_ids,      type: Array
  field :action_performed,  type: String
  field :message,           type: String
  field :snapshot,          type: Hash
  field :associated_class,  type: String
  field :child,             type: String

  # Add indexes for associated_class and associated_id since we use that as lookups below
  index({associated_class: 1, associated_id: 1}, {background: true})

  validates_presence_of :associated_id, :associated_class
  validates :action_performed, inclusion: { in: ACTIONABLE_ACTIONS, allow_nil: false }

  scope :for_associated, -> (object) { where(associated_class: object.class.to_s, associated_id: object.id) }

  def set_acting_user=(user)
    if user.is_a? String
      self.acting_user = user
    else
      self.acting_user = user.name if user.respond_to?(:name)
      self.acting_user_email = user.email if user.respond_to?(:email)
    end
  end

  def associated=(parent)
    self.associated_id = parent.id
    self.associated_class = parent.class.to_s
  end

  def actionable_action?
    ACTIONABLE_ACTIONS.include? action_performed
  end

  def non_crud_actionable_action?
    NON_CRUD_ACTIONABLE_ACTIONS.include? action_performed
  end

  def trogdir_actionable_action?
    TROGDIR_ACTIONABLE_ACTIONS.include? action_performed
  end

  def ws_actionable_actions?
    WS_ACTIONABLE_ACTIONS.include? action_performed
  end

  def to_s
    action_performed.humanize.capitalize
  end

  def changes_made
    snapshot.map do |attrib, changes|
      "#{attrib.titleize} was changed" +
        case changes.try(:length)
        when 2
          " from #{changes[0].inspect} to #{changes[1].inspect}"
        when 1
          " to #{changes.inspect}"
        end + '.'
    end
  end
end
