class Article < Edition
  include Mongoid::Document
  include Mongoid::History::Trackable
  include HasPresentationData
  include RelatesObjectTags
  include ArticlePhoto
  include Elasticsearchable

  field :body, type: String
  field :footnotes, type: String
  field :topics, type: Array
  field :audience, type: Array
  field :teaser, type: String
  field :subtitle, type: String # user customizable field
  #=> needs to match the partial name in the front end rails app
  field :article_template, type: String
  field :ws_source, type: String, default: 'WCMS'
  field :ws_id, type: String
  # Authors in ws are naught but a name so in order to ease
  #  the import process we will display a name
  field :ws_author, type: String
  field :press_release, type: Boolean
  field :imported, type: Boolean, default: false
  field :featured, type: Boolean, default: false
  # If present can be used to direct user to article's external location.
  field :external_url, type: String

  has_and_belongs_to_many :authors, class_name: 'Person', inverse_of: :articles
  belongs_to :site
  belongs_to :user
  has_and_belongs_to_many :departments
  has_and_belongs_to_many :related_people, class_name: 'Person',
                                           inverse_of: :related_articles
  has_and_belongs_to_many :site_categories
  embeds_many :gallery_photos, as: :exhibitable

  before_save :increment_ws_id

  validates_presence_of :body
  validates_uniqueness_of :slug, scope: :class
  validate :non_imported_site_presence
  validate :author_or_ws_author
  validate :site_categories_type
  validates :external_url, absolute_uri: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true,
                track_destroy: true

  scope :not_imported, -> { where imported: false }
  scope :imported, -> { where imported: true }
  scope :featured, -> { where featured: true }
  scope :by_topic, ->(topic) { where(topics: topic) }
  scope :by_category, ->(category) { where(site_category_ids: category) }
  scope :by_site, ->(site) { where(site: site) }
  scope :by_author, ->(author) { where(author_ids: author) }
  scope :by_department, ->(department) { where(department_ids: department) }

  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        any_of({title: q})
      end
    rescue RegexpError
      Article.none
    end
  end)

  def topics_string
    topics.join(', ') if topics.present?
  end

  def topics_string=(string)
    self.topics = string.to_s.split(',').map(&:titleize).map(&:strip)
  end

  def to_s
    title
  end

  # sets slug from title
  def set_slug
    return unless title?
    appendage = nil
    while ::Article.where(slug: "#{title.parameterize}#{appendage}").present?
      appendage.nil? ? appendage = 1 : appendage += 1
    end
    self.slug = "#{title.parameterize}#{appendage}"
  end

  def as_indexed_json(*)
    {
      title: title,
      slug: slug,
      body: body,
      sanitized_body: sanitized_body,
      audience: audience.to_a.map{|x| x.to_s.downcase.strip.presence }.compact,
      teaser: teaser,
      subtitle: subtitle,
      topics: topics,
      topic_slugs: topics.to_a.map(&:parameterize),
      press_release: press_release,
      author_ids: authors.map(&:id).map(&:to_s), # Use map since authors already get loaded
      author_names: authors.map(&:name), # name is a method that needs to be called
      author_slugs: authors.map(&:slug),
      site_id: site.try(:id).try(:to_s),
      categories: site_categories.to_a.map(&:to_s),
      category_slugs: site_categories.to_a.map(&:slug),
      image_x_small_url: image.x_small.url,
      image_small_url: image.small.url,
      image_medium_url: image.medium.url,
      image_large_url: image.large.url,
      image_x_large_url: image.x_large.url,
      image_xx_large_url: image.xx_large.url,
      image_url: image.url,
      publish_at: publish_at,
      external_url: external_url
    }
  end

  mapping do
    indexes "category_slugs", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
    indexes "topics", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
    indexes "topic_slugs", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
    indexes "author_slugs", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
  end

  def dont_index?
    !published?
  end

  private

  def non_imported_site_presence
    return unless !imported && site_id.blank?
    errors.add :base, 'Site is required.'
  end

  def site_categories_type
    return unless site_categories&.ne(type: 'Article').present?
    errors.add :base, 'All categories must be for articles.'
  end

  def author_or_ws_author
    return if ws_author.present? || authors.present?
    errors.add :base, 'Must have at least one author or '\
                      'a legacy author from ws.'
  end

  def increment_ws_id
    return unless ws_id.blank? && !imported
    self.ws_id = Article.where(imported: false).max(:ws_id).to_i + 1
  end

  def sanitized_body
    body.gsub(%r{</?[^>]+?>}, ' ').squeeze(' ') if body.present?
  end
end
