class Attachment
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  belongs_to :attachable, polymorphic: true

  field :name, type: String
  field :description, type: String
  field :type, type: String
  field :order, type: Integer
  mount_uploader :attachment, AttachmentUploader

  # set an ATTACHMENT_METADATA_FIELDS constant in the attachable model.
  # This will be used to dynamically create metadata fields in academic publisher
  field :metadata, type: Hash
  scope :with_metadata, ->(k,v) { where("metadata.#{k}" => v)}

  before_save :ensure_attachable_id_is_bson_object_id
  before_create :set_order

  validates :attachment, presence: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({ name: q })
      end
    rescue ::RegexpError
      Attachment.none
    end
  }

  def to_s
    self.name.presence || (attachment.present? && ::File.basename(attachment.path)) || "Attachment is uninitialized"
  end

  private

  def set_order
    self.order ||= Attachment.where(attachable_id: attachable_id).count + 1
  end

  # This was causing some weird issues with attachments being polymorphic.
  # Even though the attachment had the class and id correct, it wouldn't show up under @academic_program.attachments.
  # Keeping it for now until someone can take a closer look.
  def ensure_attachable_id_is_bson_object_id
    if attachable_id.is_a? String
      self.attachable_id = BSON::ObjectId.from_string attachable_id
    end
  end
end
