class Catalog
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  ATTACHMENT_TYPES = %w(catalog-download catalog-cover-image catalog-pdf)
  ATTACHMENT_METADATA_FIELDS = [
    FormFieldDefinition.new(name: "type", label: "Type", type: "text_field", help_text: ATTACHMENT_TYPES.join(", ")),
    FormFieldDefinition.new(name: "year", label: "Year", type: "text_field", help_text: "Example: 2013-14"),
  ]

  before_validation :set_slug

  has_many :attachments, as: :attachable, dependent: :destroy
  has_many :degree_requirements

  field :name, type: String # Ex: 2013-14 Catalog
  field :slug, type: String # Ex: 2013-14 or 2013-14-supplement
  field :full, type: Boolean, default: true
  field :external_id, type: String # used to store the external catalog system's ID for this catalog year
  field :external_url, type: String # Ex: "http://catalog.biola.edu/content.php?catoid=10&navoid=799"
  field :errata_url, type: String # used to store a link to an optional "Catalog Errata Sheet"
  field :alternate_file_download_link_text, type: String # Ex: "2013-14 Full Catalog" or "2010-11 Catalog Supplement" or "2011-12"
  field :published, type: Boolean, default: true
  field :primary, type: Boolean, default: false # used to designate that the catalog is now in the past

  validates :name, uniqueness: true, presence: true
  validates :slug, uniqueness: true, presence: true
  validates :external_id, uniqueness: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({ name: q })
      end
    rescue ::RegexpError
      Catalog.none
    end
  }

  alias :to_s :name

  def self.update_from_catalog_entry(entry_hash)
    Catalog.find_or_initialize_by(external_id: entry_hash["Catalog OID"], name: entry_hash["Catalog Name"]).tap do |cat|
      if cat.new_record?
        cat.save
      end
    end
  end

  def courses
    catalog_entries.map(&:course)
  end

  def download
    attachments.where('metadata.type' => 'catalog-download').first
  end

  def cover_image
    attachments.where('metadata.type' => 'catalog-cover-image').first
  end

  def supplement?
    !full
  end

  def link_text
    alternate_file_download_link_text || name
  end

  def disclaimer_text
    "Information in the #{name} is accurate as of the date of publication. Biola University reserves the right to make changes in policies, regulations, and fees, giving reasonable notice of changes made that may impact student requirements."
  end

  def to_s
    name
  end

  private
  def set_slug
    if name? && !slug?
      self.slug = name.parameterize
    elsif slug?
      self.slug = slug.parameterize
    end
  end
end
