# For details, see https://github.com/elasticsearch/elasticsearch-rails/blob/master/elasticsearch-model/README.md
module Elasticsearchable
  extend ActiveSupport::Concern

  included do
    include Elasticsearch::Model

    settings BuwebContentModels.config.elasticsearch_settings

    # include Elasticsearch::Model::Callbacks  # index on save/update/destroy
    #  ^ I would use this, because it should handle indexing automatically, but the problem is that
    #     the __elasticsearch__.update_document method actually checks for attributes which have updated
    #     and only push those attributes into elasticsearch. But the problem is that our as_indexed_json
    #     method returns something that doesn't map directly to the model's attributes.
    #     See: https://github.com/elasticsearch/elasticsearch-rails/issues/34  for discussion.
    after_create  { add_to_index  }
    after_update  { add_to_index }
    after_destroy { remove_from_index }

    # Overwrite this in the model
    # It needs to respond to one argument, an options hash. But we don't really care
    # about that so I am using an asterisk to accept whatever parameters
    def as_indexed_json(*)
      raise NotImplementedError, "#as_indexed_json should be overriden in #{self.class}"
      # searchable_attribute: value,
      # Make sure normalized data exists and is in this format for search results
      # normalized_data: {
      #   title: nil,
      #   subtitles: [],
      #   short_description: nil,  # this should just be a really short description.
      #   location: nil,
      #   image_url: nil,
      #   phone: nil,
      #   alternate_phone: nil,
      #   email: nil,
      # }
    end

    # This can be overridden if it should be true.
    def dont_index?
      false
    end

    # For now we are going to have everything share the same app specific index.
    index_name BuwebContentModels.config.elasticsearch_index
  end

  # This should be overridden in the models if there are any dependent indexes.
  # A dependent index is a association that should be reindexed when this model
  # is reindexed.
  # Example: [:user, :departments]
  def dependent_indexes
    []
  end

  def add_to_index
    added =
      if dont_index?
        logger.debug "This object is set to not be indexed."
        remove_from_index
        false
      else
        __elasticsearch__.index_document
        true
      end

    reindex_dependencies

    added
  end

  def remove_from_index
    begin
      __elasticsearch__.delete_document
    rescue Elasticsearch::Transport::Transport::Errors::NotFound
      # By default, elasticsearch will complain if it can't find the object you are trying to remove
      #   from the index, but we don't really care since we are trying to remove the object from the index anyways.
      logger.debug "Object was not removed from Elasticsearch because it couldn't be found"
    end

    reindex_dependencies
  end

  protected

  def reindex_dependencies
    dependent_indexes.each do |association_name|
      association = send(association_name)

      if association.respond_to? :each
        association.each do |assoc|
          assoc.add_to_index if assoc.present?
        end
      elsif !association.nil?
        association.add_to_index
      end
    end
  end
end
