##
# Adds the ability for models to be managed through permissions.
module PermissionsSubject
  extend ActiveSupport::Concern

  included do
    embeds_many :permissions

    # NOTE: elem_match doesn't merge hashes when you call it twice, the second
    # one just overwrites the first. So we have to restate the conditions in
    # `by_actor` each time, you can't just chain them together and let them
    # build off each other.
    scope :by_actor, lambda { |actor|
      elem_match({
        permissions: {
          actor_type: actor.class.to_s,
          actor_id: actor.id.to_s
        }
      })
    }

    scope :with_permission_to, lambda { |ability, actor|
      elem_match({
        permissions: {
          actor_type: actor.class.to_s,
          actor_id: actor.id.to_s,
          ability: ability
        }
      })
    }

    scope :with_any_permission_to, lambda { |ability_array, actor|
      elem_match({
        permissions: {
          actor_type: actor.class.to_s,
          actor_id: actor.id.to_s,
          ability: {
            "$in" => ability_array
          }
        }
      })
    }
  end

  def actors
    array = []
    self.permissions.order_by(:actor_type.asc).each do |p|
      obj = p.actor_type.constantize.find(p.actor_id)
      array << obj unless obj.nil?
    end
    array
  end

  def permission?(actor)
    actors.include?(actor)
  end
  alias has_permission? permission?

  def permission_to?(ability, actor)
    self.permissions.by_actor(actor).where(ability: ability).present?
  end
  alias has_permission_to? permission_to?

  def authorize!(actor, ability)
    self.new_permissions_for(actor, ability).save
  end

  def authorize(actor, ability)
    self.new_permissions_for(actor, ability).save
  end

  def unauthorize!(actor, ability)
    self.remove_permissions_for(actor, ability)
  end

  # find out if the destroy action works with the current user listed
  def unauthorize_all!(actor)
    self.permissions.where(
      actor_type: actor.class.to_s,
      actor_id: actor.id
    ).destroy_all
  end

  def new_permissions_for(object, ability = nil)
    self.permissions.new(
      actor_id: object.id,
      actor_type: object.class.to_s,
      ability: ability
    )
  end

  # find out if the destroy action works with the current user listed
  def remove_permissions_for(object, ability=nil)
    self.permissions.find_by(actor_id: object.id, ability: ability).destroy
  end
end
