require 'aasm'

module WorkFlow
  extend ActiveSupport::Concern

  included do
    include AASM
    include Wisper::Publisher
    field :aasm_state, type: String
    field :publish_sidekiq_id, type: String # TODO: Depricated
    field :archive_sidekiq_id, type: String # TODO: Depricated
    field :publish_at, type: DateTime
    field :archive_at, type: DateTime

    validate :archive_at_cannot_be_before_publish_at
    validate :only_archive

    # If the object is approved and within the publish_at timeframe,
    # move it to published, likewise if it is published but should be archived,
    # move it to archived.
    before_save :enforce_publication_times

    scope :draft, -> { any_of({aasm_state: nil}, {aasm_state: 'draft'}) }
    scope :ready_for_review, -> { where(aasm_state: 'ready_for_review') }
    scope :approved, -> { where(aasm_state: 'approved') }
    scope :published, -> { where(aasm_state: 'published') }
    scope :archived, -> { where(aasm_state: 'archived') }

    scope :publish_at_past, -> { where(:publish_at.lt => Time.zone.now) }
    scope :archive_at_future, -> { where("$or" => [{archive_at: nil}, {:archive_at.gt => Time.zone.now}]) }
    scope :archive_at_past, -> { where(:archive_at.lt => Time.zone.now) }

    # These two scopes should do the same as the corresponding instance methods below.
    scope :should_transition_to_published, -> { approved.publish_at_past.archive_at_future }
    scope :should_transition_to_archived, -> { published.archive_at_past }

    scope :by_status, -> status {
      case status.to_s
      when 'draft'
        draft
      when 'ready_for_review'
        ready_for_review
      when 'approved'
        approved
      when 'published'
        published
      when 'archived'
        archived
      end
    }

    aasm do
      state :draft, :initial => true
      state :ready_for_review
      state :approved
      state :published
      state :archived

      event :submit_for_review do
        success do
          # will only be fired when you use the auto-save (bang) method. eg. `submit_for_review!`
          broadcast_workflow_event :object_submitted_for_review
        end
        transitions from: :draft, to: :ready_for_review
      end

      event :return_to_draft do
        success do
          # will only be fired when you use the auto-save (bang) method.
          broadcast_workflow_event :object_returned_to_draft
        end
        transitions from: [:ready_for_review, :archived], to: :draft
      end

      event :approve do
        success do
          # will only be fired when you use the auto-save (bang) method.
          broadcast_workflow_event :object_approved
        end
        transitions from: [:draft, :ready_for_review], to: :approved
      end

      event :unapprove do
        success do
          # will only be fired when you use the auto-save (bang) method.
          broadcast_workflow_event :object_unapproved
        end
        transitions from: :approved, to: :draft
      end

      event :publish do
        success do
          # will only be fired when you use the auto-save (bang) method.
          broadcast_workflow_event :object_published
        end
        transitions from: [:draft, :ready_for_review, :approved], to: :published
      end

      event :archive do
        success do
          # will only be fired when you use the auto-save (bang) method.
          broadcast_workflow_event :object_archived
        end
        transitions from: :published, to: :archived
      end
    end

    def archive_at_cannot_be_before_publish_at
      errors.add(:archive_at, "can't be earlier than the publish date.") if archive_at.present? && publish_at.present? && archive_at <= publish_at
    end

    def only_archive
      errors.add(:archive_at, "can't be set without a publish date.") if archive_at.present? && publish_at.blank?
    end
  end

  def aasm_state
    self[:aasm_state] || "draft"
  end

  def aasm_transition(event)
    self.send(event) if aasm.events.include?(event)
  end

  # Only transition to published automatically like this if it has been approved
  def should_transition_to_published?
    approved? && publish_at && publish_at.past? && (archive_at.blank? || archive_at.future?)
  end

  def should_transition_to_archived?
    published? && archive_at && archive_at.past?
  end

  def enforce_publication_times
    if should_transition_to_published?
      self.publish
    elsif should_transition_to_archived?
      self.archive
    end
  end

  private

  # wrapper for Wisper::Publisher broadcast method
  def broadcast_workflow_event(event)
    broadcast(event, self.class, self.id, self.modifier.try(:class), self.modifier.try(:id))
  end
end
