class Course
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  embeds_many :catalog_entries, as: :cataloged

  field :prefix, type: String
  field :code, type: Integer
  field :name, type: String
  field :description, type: String
  field :when_offered, type: String
  field :lecture_lab_hours, type: String
  field :cross_listed_course_keys, type: String
  field :prerequisites, type: String
  field :corequisites, type: String
  field :restrictions, type: String
  field :catalog_notes, type: String
  field :fee, type: String
  field :grade_mode, type: String
  field :credits, type: String

  # TODO: Decide if we need to make courses catalog year specific... or since this is just web content, do we only keep the latest version of the course?

  alias :title :name
  alias :subject :prefix

  validates_uniqueness_of :code, scope: :prefix
  validates_presence_of :name, :prefix, :code
  validates_format_of :credits, with: /^\d(\.\d)?(-\d(\.\d)?)?$/, multiline: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :serach_course_key, -> key_string do
    key = parse_key(key_string)
    where prefix: key[:prefix], code: key[:code]
  end

  scope :custom_search, -> query {
    begin
      if query.present?
        nested_conditions = []
        # We need to take into account first and last name, which is why we split up the query
        #   and link all the words together with an '$and' filter so all the words need to match something.
        query.split.each do |q|
          fields = [:title, :prefix, :code]
          conditions = fields.map {|f| {f => f == :code ? q.to_i : ::Regexp.new("^#{q}", 'i')}}
          nested_conditions << {'$or' => conditions}
        end
        where('$and' => nested_conditions)
      end
    rescue ::RegexpError
      Course.none
    end
  }

  def course_key
    "#{subject} #{code}"
  end

  alias :course_key_string :course_key

  def course_key_string=(course_key)
    match = Course.parse_key(course_key)
    self.prefix = match[:prefix]
    self.code = match[:code]
  end

  def self.parse_key(course_key)
    match = course_key.to_s.match(/(?<prefix>[a-z]+)\s?(?<code>[0-9]+)/i)
    {prefix: match.try(:[], :prefix), code: match.try(:[], :code).try(:to_i)}
  end

  def self.by_key(course_key)
    where(parse_key(course_key)).first
  end

  def self.by_keys(*course_keys)
    course_keys = course_keys.flatten

    if course_keys.present?
      any_of(*course_keys.map { |key| parse_key(key) })
    else
      none
    end
  end

  # TODO: These catalog update methods should probably be moved into their own class
  def self.update_from_catalog_entry(entry_hash)
    # find or initialize by prefix and code; ie. BBST 103
    Catalog.update_from_catalog_entry(entry_hash)

    Course.find_or_initialize_by(prefix: entry_hash["Prefix"], code: entry_hash["Code"]).tap do |c|
      if c.new_record?
        entry = c.catalog_entries.new raw_data: entry_hash, external_id: entry_hash["Course OID"], external_catalog_id: entry_hash["Catalog OID"]
        c.update_from_entry(entry)
      else
        # does this catalog entry already exist? match on "Catalog OID" and "Course OID"
        entry = c.catalog_entries.find_or_initialize_by(external_catalog_id: entry_hash["Catalog OID"], external_id: entry_hash["Course OID"])
        entry.raw_data = entry_hash
        c.update_from_entry(entry)
      end
    end
  end

  def update_from_latest_entry
    entry = catalog_entries.order_by(:last_modified_at.desc).first # Not sure this will exist yet to query
    update_from_entry(entry)
  end

  def update_from_entry(entry)
    self.prefix                   = entry.raw_data["Prefix"]
    self.code                     = entry.raw_data["Code"]
    self.name                     = entry.raw_data["Name"]
    self.description              = entry.raw_data["Description (Rendered)"]
    self.when_offered             = entry.raw_data["When Offered"]
    self.lecture_lab_hours        = entry.raw_data["Lecture/Lab Hours"]
    self.cross_listed_course_keys = entry.raw_data["Cross-listed"].try(:gsub, /[.]{1}$/, '')
    self.prerequisites            = entry.raw_data["Prerequisite(s)"].try(:gsub, /[.]{1}$/, '')
    self.corequisites             = entry.raw_data["Corequisite(s)"].try(:gsub, /[.]{1}$/, '')
    self.restrictions             = entry.raw_data["Restriction(s)"]
    self.catalog_notes            = entry.raw_data["Note(s)"]
    self.fee                      = entry.raw_data["Fee"].try(:gsub, /[.]{1}$/, '')
    self.grade_mode               = entry.raw_data["Grade Mode"].try(:gsub, /[.]{1}$/, '')
    self.credits                  = entry.raw_data["Credit(s)"].try(:gsub, /[.]{1}$/, '')
    if entry.raw_data["Prefix"] && entry.raw_data["Code"] && entry.raw_data["Name"]
      self.save(validate: false)
    else
      raise "Course must contain a prefix, code, and name."
    end
  end

  def to_s
    name
  end

end
