class Department
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include PermissionsSubject
  include BiolaPhone
  include Elasticsearchable
  include RelatesObjectTags
  include HasPresentationData
  include ActsAsWebPage
  include HasPages

  LABEL_TYPES = [ 'department', 'division', 'service' ]
  CATEGORIES = ['Academic Departments', 'Campus Services']

  has_many :child_departments, class_name: 'Department'
  belongs_to :parent_department, class_name: 'Department'

  has_and_belongs_to_many :academic_subjects
  has_and_belongs_to_many :events
  has_and_belongs_to_many :articles
  has_and_belongs_to_many :publications

  embeds_many :gallery_photos, as: :exhibitable
  embeds_many :memberships, as: :unitable
  embeds_many :social_media
  embeds_many :department_contacts
  embeds_one :address
  belongs_to :campus_location

  field :title, type: String # user customizable field
  field :slug, type: String
  field :label, type: String, default: 'Department'
  field :aliases, type: Array
  field :hr_title, type: String # managed by HR/Payroll data
  field :service_content, type: String
  field :description, type: String
  field :short_description, type: String # NOT BEING USED
  field :location, type: String
  field :location_details, type: String
  field :website, type: String
  field :phone, type: String
  field :email, type: String
  field :fax, type: String
  field :open_hours, type: String  # html formated description of open hours
  field :published, type: Boolean, default: true
  field :forms_department_id, type: Integer  # For relating to a department in Forms.
  field :categories, type: Array

  mount_uploader :logo, GenericPhotoUploader
  mount_uploader :banner, GenericPhotoUploader

  before_validation :set_slug, unless: :slug?
  validates :title, :slug, presence: true, uniqueness: true
  validates :hr_title, uniqueness: true, allow_blank: true
  validates :website, absolute_uri: true, allow_blank: true

  accepts_nested_attributes_for :department_contacts, reject_if: :all_blank, allow_destroy: true
  accepts_nested_attributes_for :social_media, reject_if: :all_blank, allow_destroy: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :published, -> { where published: true }

  scope :by_last_change, -> last_change {
    return scoped if last_change.nil?
    where('updated_at' => { '$gt' => Time.parse(last_change.to_s) })
  }

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q}, {aliases: q})
      end
    rescue ::RegexpError
      Department.none
    end
  }

  def people
    memberships.map(&:person)
  end

  def to_s
    title
  end

  # This should match by hr_title or title or create a new department
  def self.first_or_create_by_hr_title(hr_title)
    best_match = Department.where(hr_title: hr_title).first
    ok_match = Department.where(title: hr_title).first
    if best_match || ok_match
      best_match ? best_match : ok_match.update_attributes(hr_title: hr_title) # TODO: make this more readable and isolate responsibilities
    else
      Department.create!(title: hr_title, hr_title: hr_title) # will throw an error if this fails
    end
  end

  def aliases_string
    self.aliases.join('| ') if aliases.present?
  end

  # TODO: Adam wants this changed to be in the controllers
  def aliases_string=(string)
    self.aliases = string.to_s.split('|').map(&:strip)
  end

  def as_indexed_json(*)
    {
      slug: slug,
      title: title,
      title_edge: title,
      aliases: aliases,
      description: description,
      location: location,
      location_details: location_details,
      website: website,
      phone: full_biola_phone_number,
      email: email,
      fax: fax,
      categories: categories,
      normalized_data: {
        title: title,
        subtitles: [],
        short_description: short_description,
        location: location,
        image_url: nil,
        phone: full_biola_phone_number,
        alternate_phone: nil,
        published: published,
        email: email,
        page_url: page_url
      },
      is_public: is_public?  # if this was false it would be hidden from unauthenticated users.
    }
  end

  mapping do
    # TODO: Look into using multi field type for this.
    #  http://www.elasticsearch.org/guide/en/elasticsearch/reference/0.90/mapping-multi-field-type.html#mapping-multi-field-type
    indexes "title", type: 'string', fields: {
      raw:  { type: 'string', index: 'not_analyzed' },
      edge: { type: 'string', analyzer: 'edge_ngram_analyzer' }
    }
    indexes "title_edge", type: 'string', analyzer: 'edge_ngram_analyzer'
    indexes "normalized_data.title", type: 'string', index: 'not_analyzed'
    indexes "categories", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
  end

  def dependent_indexes
    [:people, :events]
  end

  def dont_index?
    !published
  end

  def is_public?
    true
  end

  # sets slug from title
  def set_slug
    self.slug = title.parameterize if title?
  end
end
