class Event < Edition
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include PermissionsSubject
  include RelatesObjectTags
  include HasPresentationData
  include HasPages

  LOCATION_TYPES = %w[pending on-campus off-campus online].freeze

  before_save :set_ws_id
  before_save :cache_dates

  # In case published status or event details change
  after_update { event_occurrences.each(&:add_to_index) }

  # IMPORTIANT: Any time you remove or change one of these fields be sure to
  #  copy over any of the data that needs to be maintained and then run
  #  Event.each { |event| event.unset(:type) }
  #  where :type is the field that was changed or removed
  field :subtitle, type: String # user customizable field
  field :location_type, type: String, default: 'pending'
  # custom_campus_location will also be used for ws locations,
  # which will not always be on campus
  field :custom_campus_location, type: String
  field :online_location, type: String
  field :online_location_name, type: String
  field :categories, type: String
  field :contact_name, type: String
  field :contact_email, type: String
  field :contact_phone, type: String
  field :ws_source, type: String
  field :ws_id, type: String
  field :paid, type: Boolean, default: false
  field :imported, type: Boolean, default: false
  field :featured, type: Boolean, default: false
  field :registration_link, type: String
  field :registration_button_text, type: String, default: "Register Now"

  # DEPRECATED: These fields were duplicated from ws and will no longer
  #  be used. However their existing values will still need to be retained.
  field :admission_info, type: String
  field :audience, type: Array
  field :map_url, type: String
  field :registration_info, type: String
  field :sponsor, type: String
  field :sponsor_site, type: String
  field :teaser, type: String

  # Used in pair to match an event to some imported event
  # These shouldn't get used by anything other than the import task
  # and will get removed once we have finished migrating to the
  # new events site.
  field :import_source, type: String
  field :import_id, type: String
  # Optional flag to prevent importing if you need to manually override
  # certain imported fields.
  field :skip_import, type: Boolean, default: false

  # start_date and end_date are will now cache the values of the first
  # occurrence start_time and the last occurrence end_time
  # These fields probably shouldn't get displayed, just used to sort.
  # See `future` and `past` scopes
  field :start_date, type: DateTime
  field :end_date, type: DateTime

  mount_uploader :image, EventPhotoUploader

  validates_presence_of :title, :description, :image
  validates_uniqueness_of :slug, scope: :class
  validates_inclusion_of :location_type, in: LOCATION_TYPES
  validates :online_location, absolute_uri: true, allow_blank: true
  validates :registration_link, absolute_uri: true, allow_blank: true
  validate :non_imported_site_presence
  validate :site_categories_type
  validate :at_least_one_occurrence

  has_and_belongs_to_many :departments
  has_and_belongs_to_many :groups
  has_and_belongs_to_many :site_categories
  has_and_belongs_to_many :external_sponsors, {
    class_name: 'Organization',
    inverse_of: :sponsored_events
  }
  has_and_belongs_to_many :event_collections
  embeds_one :address
  embeds_many :links
  embeds_many :tickets
  # cascade_callbacks so elasicseach callbacks will run
  embeds_many :event_occurrences, cascade_callbacks: true
  accepts_nested_attributes_for :event_occurrences, {
    allow_destroy: true,
    reject_if: ->(o) { o[:start_time].blank? && o[:end_time].blank? }
  }
  belongs_to :site
  belongs_to :user
  # This is a one sided relationship because campus locations
  #  dont need to know which events have been held there
  belongs_to :campus_location

  track_history on: :fields, track_create: true, track_update: true,
                track_destroy: true

  scope :not_imported, -> { where imported: false }
  scope :imported, -> { where imported: true }
  scope :featured, -> { where featured: true }
  scope :by_category, ->(category) { where(site_category_ids: category) }
  scope :by_site, ->(site) { where(site: site) }
  scope :by_department, ->(department) { where(department_ids: department) }
  scope :by_group, ->(group) { where(group_ids: group) }
  scope :past, -> { lt end_date: Time.zone.today.beginning_of_day }
  scope :future, -> { gte end_date: Time.zone.today.beginning_of_day }
  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        any_of({title: q}, subtitle: q)
      end
    rescue RegexpError
      Event.none
    end
  end)

  def to_s
    title.to_s
  end

  def location_pending?
    location_type == 'pending'
  end

  def location
    if location_type == 'on-campus'
      custom_campus_location.presence || campus_location
    elsif location_type == 'off-campus'
      address
    end
  end

  # sets slug from title
  def set_slug
    return unless title?
    appendage = nil
    while ::Event.where(slug: "#{title.parameterize}#{appendage}").present?
      appendage.nil? ? appendage = 1 : appendage += 1
    end
    self.slug = "#{title.parameterize}#{appendage}"
  end

  def unending_event?
    end_date.blank?
  end

  def day_span
    unending_event? ? 1 : (end_date.to_date - start_date.to_date + 1).to_i
  end

  def single_day_event?
    day_span == 1
  end

  def multiple_day_event?
    day_span > 1
  end

  def projected_ending
    unending_event? ? start_date.end_of_day : end_date
  end

  def clone
    new_event = super
    new_event.set_slug
    new_event.address = self.address.clone
    new_event.campus_location = self.campus_location.clone if self.campus_location
    new_event.write_attributes(
      ws_id: nil,
      aasm_state: nil,
      publish_at: nil,
      archive_at: nil,
      start_date: nil,
      end_date: nil,
    )
    return new_event
  end

  # Just index all the embedded event_occurrences
  def add_to_index
    event_occurrences.each(&:add_to_index)
  end

  # Will always return an event occurrence. It will try to show the next upcomming
  # one otherwise it will show the last occurrence if there are no more future dates.
  def next_occurrence
    @next_occurrence ||= get_next_occurrence
  end
  def get_next_occurrence
    # Return the next upcomming occurrences
    event_occurrences.asc(:start_time).each do |o|
      return o if o.start_time > Time.now || (o.end_time && o.end_time > Time.now)
    end

    # Otherwise return the last one
    return event_occurrences.asc(:start_time).last
  end

  def get_first_occurrence
    event_occurrences.asc(:start_time).first
  end

  def get_last_occurrence
    # I'm going to assume no wierd overlapping dates, where it starts
    #  before another occurrence but ends after, mainly since end_date
    #  is not guaranteed to exist.
    event_occurrences.asc(:start_time).last
  end

  private

  def non_imported_site_presence
    if !imported && site_id.blank?
      errors.add :site, 'is required.'
    end
  end

  def site_categories_type
    if site_categories.present? && site_categories.ne(type: "Event").present?
      errors.add :site_categories, 'must be for events.'
    end
  end

  def at_least_one_occurrence
    if event_occurrences.length == 0
      errors.add :base, "Valid time is required."
    end
  end

  # max wcms id and wcms-music id. Both sources will share the same id space
  # self.ws_id = ::Event.any_of(
  #  {ws_source: 'WCMS'},
  #  {ws_source: 'WCMS-Music'}
  # ).pluck(:ws_id).map(&:to_i).max + 1
  def set_ws_id
    if ws_id.blank? && !imported
      self.ws_id = ::Event.where(imported: false).max(:ws_id).to_i + 1
    end
  end

  # We will cache start and end dates for performance reasons
  def cache_dates
    first_occurrence = get_first_occurrence
    last_occurrence = get_last_occurrence
    return if first_occurrence.nil? || last_occurrence.nil?

    self.start_date = first_occurrence.start_time
    self.end_date = last_occurrence.end_time || last_occurrence.start_time
  end
end
