class EventOccurrence
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include Elasticsearchable

  embedded_in :event

  field :start_time, type: DateTime
  field :end_time, type: DateTime
  field :all_day, type: Boolean, default: false

  validates :start_time, presence: true

  track_history on: :fields,
                track_create: true,
                track_update: true,
                track_destroy: true,
                scope: [ :event ]

  validate :ends_after_it_starts

  def ends_after_it_starts
    errors.add :end_time, 'should be after the start time.' if start_time && end_time && (end_time < start_time)
  end

  def day_span
    end_time.blank? ? 1 : (end_time.to_date - start_time.to_date + 1).to_i
  end

  def single_day_event?
    day_span == 1
  end

  def multiple_day_event?
    day_span > 1
  end

  def projected_ending
    end_time.blank? ? start_time.end_of_day : end_time
  end

  def as_indexed_json(*)
    {
      event_id: event.id.to_s,
      slug: event.slug,
      title: event.title,
      subtitle: event.subtitle,
      description: event.description,
      page_url: event.page_url,
      start_time: start_time,
      end_time: end_time,
      start_time_index: start_time.to_i,
      end_time_index: (end_time || start_time).to_i,  # in case end_time is nil, use start_time
      all_day: all_day,
      year: start_time.year,
      month: start_time.month,
      day: start_time.day,
      day_span: day_span,
      multiple_day_event: multiple_day_event?,
      organizations: event.departments.to_a.map(&:title) + event.groups.to_a.map(&:title),
      organizations_slugs: event.departments.to_a.map(&:slug) + event.groups.to_a.map(&:slug),
      event_collections_slugs: event.event_collections.to_a.map(&:slug),
      teaser: event.teaser,
      audience: event.audience.to_a.map{|x| x.to_s.downcase.strip.presence }.compact,
      categories: event.site_categories.to_a.map(&:to_s),
      contact_name: event.contact_name,
      contact_email: event.contact_email,
      contact_phone: event.contact_phone,
      admission_info: event.admission_info,
      registration_info: event.registration_info,
      map_url: event.map_url,
      image_medium_url: event.image.medium.url,
      image_small_url: event.image.small.url,
      image_thumb_url: event.image.thumb.url,
      image_url: event.image.url,
      featured: event.featured,
      external_sponsors: event.external_sponsors.to_a.map(&:title),
      location: event.location.to_s
    }
  end

  mapping do
    indexes "organizations_slugs", type: 'string', index: 'not_analyzed'
    indexes "event_collections_slugs", type: 'string', index: 'not_analyzed'
    indexes "categories", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
    indexes "audience", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
  end

  def dont_index?
    !event.published?
  end

end
