require 'uri'

class Feature
  include Mongoid::Document
  include Mongoid::History::Trackable
  include PermissionsSubject
  include RelatesObjectTags
  include WorkFlow

  TYPES = %w[content_only link video audio].freeze
  LAYOUTS = %w[full split].freeze
  IMAGE_TYPES = %w[body background].freeze

  belongs_to :feature_location
  field :title, type: String
  field :body, type: String
  field :call_to_action_link, type: String
  field :call_to_action_text, type: String
  field :type, type: String
  field :layout, type: String, default: 'full'
  field :image_type, type: String, default: ''
  field :order, type: Integer

  # Slideshow/background images
  mount_uploader :image, GenericPhotoUploader
  mount_uploader :mobile_image, GenericPhotoUploader
  field :image_alternative_text, type: String

  # Audio and video fields
  field :hd_video_url, type: String
  field :standard_video_url, type: String
  field :audio_url, type: String
  field :media_thumbnail_url, type: String

  before_create :set_order

  validates :type, inclusion: { in: TYPES }
  validates :layout, inclusion: { in: LAYOUTS }
  validates :image_type, inclusion: { in: IMAGE_TYPES }, allow_blank: true
  validates :title, :feature_location_id, presence: true

  # image alternative text validation
  validates :image_alternative_text, presence: true,
                                     if: proc { |f| f.image.present? }

  # url validation
  validates :call_to_action_link, absolute_uri: true, allow_blank: true
  validates :hd_video_url, absolute_uri: true, allow_blank: true
  validates :standard_video_url, absolute_uri: true, allow_blank: true
  validates :audio_url, absolute_uri: true, allow_blank: true
  validates :media_thumbnail_url, absolute_uri: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :by_feature_location, -> feature_location { where(feature_location: feature_location) }

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q})
      end
    rescue ::RegexpError
      Feature.none
    end
  }

  def set_order
    self.order ||= Feature.where(feature_location_id: feature_location_id).count + 1
  end

  def to_s
    title
  end
end
