require 'uri'
class FeatureLocation
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  has_many :features
  belongs_to :site
  field :title, type: String
  field :url, type: String  # depricating url in favor of path
  field :path, type: String # path is now required instead of url
  field :preferred_image_height, type: Integer
  field :preferred_image_width, type: Integer
  field :slug, type: String

  before_validation :set_slug, unless: :slug?

  validates_uniqueness_of :slug, scope: [:site_id, :path]
  validates_uniqueness_of :title, scope: :site_id
  validates_format_of :slug, with: /\A[\w\-]+\Z/i
  validates_presence_of :title, :site_id, :slug, :path
  validate :url_formatting
  validate :preferred_image
  validates_numericality_of :preferred_image_width, :preferred_image_height, allow_nil: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q})
      end
    rescue ::RegexpError
      FeatureLocation.none
    end
  }

  def to_s
    "#{site.title}: #{title}"
  end

  def title_with_dimensions
    preferred_image_width.present? ? "#{title} (#{preferred_image_width}x#{preferred_image_height})" : title
  end

  private
  def preferred_image
    if (preferred_image_height.blank? && !preferred_image_width.blank?) || (!preferred_image_height.blank? && preferred_image_width.blank?)
      errors.add :base, 'Both preferred image height and width need to be filled in.'
    end
  end

  protected
  def set_slug
    self.slug = title.parameterize if title?
  end

  def url_formatting
    if url.present?
      unless (uri?(url) || valid_relative_url?(url))
        errors.add :url, 'either needs to be a full url or a valid relative url.'
      end
    end
  end

  def uri?(string)
    uri = URI.parse(string)
    %w( http https ).include?(uri.scheme)
  rescue URI::BadURIError
    false
  rescue URI::InvalidURIError
    false
  end

  def valid_relative_url?(string)
    string.starts_with?('/')
  end

end
