class Group
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include PermissionsSubject
  include BiolaPhone
  include Elasticsearchable
  include RelatesObjectTags
  include HasPresentationData
  include ActsAsWebPage
  include HasPages

  # TODO: change this to 'Governing Documents' and fix all the current attachments with the type of 'Governing Document'
  ATTACHMENT_TYPES = ['Minutes', 'Governing Document', 'Supporting Documents', 'Forms'].sort
  CATEGORIES = ['Employee Groups', 'Student Clubs', 'Other']

  embeds_many :memberships, as: :unitable
  has_many :attachments, as: :attachable
  has_and_belongs_to_many :events

  field :title, type: String
  field :slug, type: String
  field :aliases, type: Array
  field :description, type: String
  field :body, type: String
  field :short_description, type: String
  field :email, type: String
  field :phone, type: String
  field :website, type: String
  field :fax, type: String
  field :published, type: Boolean, default: true
  field :forms_department_id, type: Integer  # For relating to a department in Forms.
  field :categories, type: Array

  alias :to_s :title

  before_validation :set_slug, unless: :slug?
  validates :title, :slug, presence: true, uniqueness: true
  validates :email, uniqueness: true, allow_blank: true
  validates :website, absolute_uri: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :published, -> { where published: true }

  scope :by_last_change, -> last_change {
    return scoped if last_change.nil?
    where('updated_at' => { '$gt' => Time.parse(last_change.to_s) })
  }

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q}, {aliases: q})
      end
    rescue ::RegexpError
      Group.none
    end
  }

  ATTACHMENT_TYPES.each do |type|
    define_method(type.parameterize('_').pluralize) do
      attachments.where(type: type)
    end
  end

  def people
    memberships.map(&:person)
  end

  def aliases_string
    self.aliases.join('| ') if aliases.present?
  end

  # TODO: Adam wants this changed to be in the controllers
  def aliases_string=(string)
    self.aliases = string.to_s.split('|').map(&:strip)
  end

  def as_indexed_json(*)
    {
      slug: slug,
      title: title,
      description: description,
      email: email,
      phone: full_biola_phone_number,
      aliases: aliases,
      fax: fax,
      categories: categories,
      normalized_data: {
        title: title,
        subtitles: [],
        short_description: short_description,
        location: nil,
        image_url: nil,
        phone: full_biola_phone_number,
        alternate_phone: nil,
        published: published,
        email: email
      },
      is_public: is_public?  # if this was false it would be hidden from unauthenticated users.
    }
  end

  mapping do
    indexes "normalized_data.title", type: 'string', index: 'not_analyzed'
    indexes "categories", type: 'string', fields: {
      raw: {type: 'string', index: 'not_analyzed'}
    }
  end

  def dependent_indexes
    [:events]
  end

  def dont_index?
    !published
  end

  def is_public?
    true
  end

  # sets slug from title
  def set_slug
    self.slug = title.parameterize if title?
  end
end
