require 'uri'

class ImportantDate
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  field :title, type: String
  field :url, type: String # for optional link to external page
  field :start_date, type: Date
  field :end_date, type: Date # optional for multiday event
  field :deadline, type: Boolean, default: false
  field :categories, type: Array, default: []

  # Used in pair to match an important date to some imported event
  # These shouldn't get used by anything other than the import task
  # and will get removed once we have finished migrating to the
  # new events site.
  field :import_source, type: String
  field :import_id, type: String
  # Optional flag to prevent importing if you need to manually override
  # certain imported fields.
  field :skip_import, type: Boolean, default: false
  field :imported, type: Boolean, default: false

  embeds_one :audience_collection

  has_and_belongs_to_many :calendars # optional relationship

  validates :title, presence: true
  validates :start_date, presence: true
  validate :end_date_after_start_date
  validates :url, absolute_uri: true, allow_blank: true

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  def to_s
    title
  end

  def days_of_week(format=:long)
    formatter = (format == :short) ? '%a' : '%A'
    start_date.strftime(formatter) +
    (end_date ? " - #{end_date.strftime(formatter)}" : "")
  end

  def date_string(format=:long)
    formatter = (format == :short) ? '%b' : '%B'
    if end_date
      if start_date.month == end_date.month
        start_date.strftime("#{formatter} %-d") + " - " + end_date.strftime("%-d")
      else
        start_date.strftime("#{formatter} %-d") + " - " + end_date.strftime("#{formatter} %-d")
      end
    else
      start_date.strftime("#{formatter} %-d")
    end
  end

  scope :by_calendar, -> calendar { where(calendar_ids: calendar) }
  scope :by_category, -> category { where(categories: category) }
  scope :by_audience, -> audience { where(audiences: audience) }
  scope :future_dates, -> { gte(start_date: Time.now) }
  scope :is_a_deadline, -> { where(deadline: true) }
  scope :by_last_change, -> last_change {
    return scoped if last_change.nil?
    where('updated_at' => { '$gte' => Time.parse(last_change.to_s) })
  }
  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        any_of({title: q}, {audiences: q}, {categories: q})
      end
    rescue ::RegexpError
      ImportantDate.none
    end
  }

  private

  def end_date_after_start_date
    errors.add :end_date, 'must be after start date' if end_date && end_date <= start_date
  end

end
