class Membership
  include Mongoid::Document
  include Mongoid::History::Trackable

  embedded_in :unitable, polymorphic: true
  belongs_to :person
  field :role, type: Array
  field :team, type: Array, default: []
  field :published, type: Boolean, default: false
  field :order, type: Integer

  validates :person, presence: true
  validates_uniqueness_of :person, scope: :unitable

  before_create :set_order
  after_destroy :reindex_person # so that this removal gets indexed into elasticsearch for the person

  track_history on: :fields,
                track_create: true,
                track_update: true,
                track_destroy: true,
                scope: :unitable

  scope :published, -> { where(published: true) }
  scope :role_or_team_search, -> query {
    begin
      if query.present?
        nested_conditions = []
        # We need to take into account first and last name, which is why we split up the query
        #   and link all the words together with an '$and' filter so all the words need to match something.
        query.split.each do |q|
          fields = [:role, :team]
          conditions = fields.map {|f| {f => ::Regexp.new("^#{q}", 'i')}}
          nested_conditions << {'$or' => conditions}
        end
        where('$and' => nested_conditions)
      end
    rescue ::RegexpError
      Membership.none
    end
  }

  def set_order
    self.order ||= (unitable.memberships.count + 1) if unitable?
  end

  def reindex_person
    person.try(:add_to_index)
  end
end
