class Menu
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  field :title, type: String
  field :url, type: String # The url you go to when you click on the header.
  # TODO: are slugs really needed? I can't think of any need for them.
  field :slug, type: String # Maintained by the user, not set from the title
  field :menu_links, type: Array, default: []

  before_validation :reset_slug

  belongs_to :site
  has_many :page_editions

  validates_presence_of :title
  validates_uniqueness_of :slug, allow_blank: true

  validate :url_formatting
  validate :menu_links_json_validation

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true


  def to_s
    title
  end

  def menu_links_json
    # Used cached data if it is there, in case there was an error when saving
    @invalid_menu_links_json || JSON.pretty_generate(menu_links)
  end

  def menu_links_json=(json)
    begin
      # Cache json in case there is a parsing error
      @invalid_menu_links_json = json

      self.menu_links = JSON.parse(json)

      # Clear cache, since json was parsed safely
      @invalid_menu_links_json = nil
    rescue JSON::ParserError
      return false
    end
  end

  scope :by_site, -> site { where(site_id: site) }
  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        any_of({title: q},{slug: q})
      end
    rescue ::RegexpError
      Menu.none
    end
  }

  private
  def reset_slug
    self.slug = nil if self.slug == ''
  end

  def menu_links_json_validation
    errors.add(:base, "JSON is not valid") if @invalid_menu_links_json.present?
  end

  def url_formatting
    if url.present?
      unless (uri?(url) || valid_relative_url?(url))
        errors.add :url, 'either needs to be a full url or a valid relative url.'
      end
    end
  end

  def uri?(string)
    uri = URI.parse(string)
    %w( http https ).include?(uri.scheme)
  rescue URI::BadURIError
    false
  rescue URI::InvalidURIError
    false
  end

  def valid_relative_url?(string)
    string.starts_with?('/')
  end
end
