class PageEdition < Edition
  include Mongoid::History::Trackable
  include HasPresentationData
  include RelatesObjectTags
  include PermissionsSubject
  include Elasticsearchable
  include StringAccessorForArray

  AVAILABLE_SOURCE_TYPES = %w[
    AcademicSubject
    AcademicProgram
    Concentration
    Department
    Event
    EventCollection
    Group
  ].freeze

  ATTACHMENT_METADATA_FIELDS = [
    FormFieldDefinition.new(
      name: 'type',
      label: 'Type',
      type: 'text_field',
      help_text: 'page-attachment'
    )
  ].freeze

  after_save :update_the_source
  after_destroy :unset_source

  # This will be used for building out breadcrumbs and help organize pages
  has_many :child_pages, class_name: 'PageEdition'
  belongs_to :parent_page, class_name: 'PageEdition'
  belongs_to :site
  belongs_to :menu
  belongs_to :source, polymorphic: true

  has_many :attachments, as: :attachable
  embeds_one :audience_collection
  embeds_one :redirect
  has_and_belongs_to_many :site_categories #(support page topics / categories)
  has_and_belongs_to_many :departments
  # embeds_many :parts

  # field :use_parts, type: Boolean, default: false # use parts instead of body
  # inherits .title() from Edition
  # inherits .slug() from Edition
  field :page_template, type: String #=> needs to match the partial name in the front end rails app
  field :topics, type: Array, default: [] # used for grouping like pages
  field :keywords, type: Array, default: [] # used for searching
  field :body, type: String
  field :gather_content_id, type: Integer

  field :page_layout, type: String

  # Provides methods to read and write these arrays as comma seperated strings
  string_accessor_for_array :topics
  string_accessor_for_array :keywords

  validates_uniqueness_of :slug, scope: :site_id
  validates_presence_of :site_id

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :redirected, -> { where(:'redirect.destination'.ne => '', :'redirect.destination'.exists => true) }
  scope :by_site, -> site { where(site_id: site) }
  scope :by_page_template, -> template { where(page_template: template) }
  scope :by_last_change, -> last_change {
    return scoped if last_change.nil?
    where('updated_at' => { '$gte' => Time.parse(last_change.to_s) })
  }
  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        # Wrap in an and to avoid colliding with other "or" statements.
        where("$and" => [{"$or" => [{title: q}, {slug: q}]}])
      end
    rescue ::RegexpError
      PageEdition.none
    end
  }


  def redirected?
    self.redirect.present? && self.redirect.destination.present?
  end

  def primary_source_page?
    source.try(:primary_page_id) == id.to_s
  end

  def update_the_source
    if source && primary_source_page?
      # Don't set page_url for unpublished pages
      if !self.published?
        source.update_attributes(page_url: nil)
      elsif source.page_url != self.url
        source.update_attributes(page_url: url)
      end
    end
  end

  def source_id=(s_id)
    begin
      s_id = BSON::ObjectId.from_string(s_id) if s_id.present?
      super(s_id)
    rescue BSON::ObjectId::Invalid
      logger.error "Invalid source_id '#{s_id}' being set for PageEdition '#{id}'"
    end
  end

  def unset_source
    if source && primary_source_page?
      source.primary_page = nil
      source.save
    end
  end

  def url
    if redirected?
      self.redirect.destination
    else
      "#{self.site.url}/#{self.slug}"
    end
  end

  def as_indexed_json(*)
    {
      title: title,
      slug: slug,
      url: url,
      site_id: site_id.to_s,
      description: description,
      body: sanitized_body,
      topics: topics,
      keywords: keywords,
      parent_page_id: parent_page_id.to_s,
      status: aasm_state,
      published: published?,
      departments: departments.to_a.map(&:title),
      department_slugs: departments.to_a.map(&:slug),
      department_aliases: departments.to_a.map(&:aliases).flatten,
      categories: site_categories.to_a.map(&:to_s)
    }
  end

  mapping do
    indexes "title", type: 'string', fields: {
      raw:  { type: 'string', index: 'not_analyzed' },
      edge: { type: 'string', analyzer: 'edge_ngram_analyzer' }
    }
  end

  def dont_index?
    !published?
  end

  def sanitized_body
    body.gsub(%r{</?[^>]+?>}, ' ').squeeze(' ') if body.present?
  end

  def enabled_roles
    [['Editor', :edit], ['Publisher', :publish]]
  end
end
