##
# Person
class Person
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include TrogdirPerson
  include Student
  include Employee
  include Alumnus
  include PermissionsSubject
  include Elasticsearchable
  include BiolaPhone
  include RelatesObjectTags

  after_save :update_bio_slug

  # TODO: should become the value of
  # self&.pluck(:affiliations)&.flatten!&.compact!&.uniq!&.sort!
  ALL_AFFILIATIONS = %i[
    accepted_student
    advisor
    alumnus
    contractor
    employee
    faculty
    faculty_emeritus
    friend
    research_fellow
    student
    student_worker
    trustee
    volunteer
  ].freeze

  GENDERS = %i[male female].freeze
  FACULTY_STATUSES = %i[full_time adjunct emeritus].freeze
  STUDENT_ROLES = %i[accepted_student student student_worker].freeze

  has_one :user
  has_many :bio_editions, validate: false
  has_many :custom_profiles
  has_many :profile_photos
  has_many :external_articles,
           class_name: 'ExternalArticle',
           inverse_of: :author
  embeds_many :social_media
  has_and_belongs_to_many :articles, class_name: 'Article', inverse_of: :authors
  has_and_belongs_to_many :related_articles,
                          class_name: 'Article',
                          inverse_of: :related_people

  # Names
  field :slug, type: String
  field :first_name, type: String # not user changeable
  field :preferred_name, type: String # not user changeable
  field :middle_name, type: String # not user changeable
  field :last_name, type: String # not user changeable
  field :previous_last_name, type: String
  field :display_name, type: String

  # Personal contact fields
  field :personal_email, type: String
  field :personal_phone, type: String
  field :personal_website, type: String # Deprecated! Use the one in BioEdition

  # Demographic
  field :gender, type: Symbol # not user changeable
  field :birth_date, type: Date # not user changeable

  # Groups and permissions
  field :affiliations, type: Array # not user changeable
  field :groupings, type: Array
  field :faculty_status, type: Symbol # TODO: who sets this?
  field :published, type: Boolean, default: true

  field :profile_photo_id, type: String

  # Denotes that the person MAY be publicly viewable (see is_public?)
  field :publicly_viewable, type: Boolean, default: false

  accepts_nested_attributes_for :social_media, reject_if: :all_blank,
                                               allow_destroy: true
  accepts_nested_attributes_for :user, update_only: true

  track_history on: :fields,
                track_create: true,
                track_update: true,
                track_destroy: true

  scope :affiliations, ->(query) { all affiliations: query }
  scope :published, -> { where published: true }
  scope :has_a_bio, -> { where :_id.in => BioEdition.pluck(:person_id) }
  scope :students, -> { where affiliations: 'student' }

  scope :by_last_change, (lambda do |last_change|
    return scoped if last_change.nil?
    where('updated_at' => { '$gt' => Time.parse(last_change.to_s) })
  end)

  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        nested_conditions = []
        # We need to take into account first and last name, which is why we split up the query
        #   and link all the words together with an '$and' filter so all the words need to match something.
        query.split.each do |q|
          fields = %i[first_name preferred_name last_name biola_email]
          # Support multi word names like "Van Houten".
          # So look for start of string or space
          conditions = fields.map { |f| { f => ::Regexp.new("(^| )#{::Regexp.escape(q)}", 'i') } }
          nested_conditions << { '$or' => conditions }
        end
        where('$and' => nested_conditions)
      end
    rescue ::RegexpError
      Person.none
    end
  end)

  def groups(published_only: true)
    conditions = published_only ? { published: true } : {}

    Group.where({ 'memberships.person_id' => id }.merge conditions)
  end

  def departments(published_only: true)
    if published_only
      Department.elem_match(memberships: {person_id: id, published: true}).where(published: true)
    else
      Department.where({'memberships.person_id' => id})
    end
  end

  def group_memberships
    groups.map do |g|
      g.memberships.to_a.find { |m| m.person_id == id }
    end.compact
  end

  def department_memberships
    departments.map do |d|
      d.memberships.to_a.find { |m| m.person_id == id }
    end.compact
  end

  def bio_edition
    @bio_edition ||= bio_editions.published.first
  end

  def custom_profile
    @custom_profile ||= custom_profiles.published.first
  end

  def profile_photo
    @profile_photo ||= ProfilePhoto.where(id: profile_photo_id).first if profile_photo_id
  end

  def id_card_image(size = :medium)
    # I want to use a size up since id_card_images come really small.
    size = :large if size == :medium
    size = :medium if %i[small thumb].include? size
    if biola_id.present?
      digest = Digest::MD5.hexdigest(biola_id.to_s)
      "https://apps.biola.edu/idphotos/#{digest}_#{size}.jpg"
    end
  end

  def profile_photo_url(size=:small)
    # use primary profile_photo if there is one, otherwise use the id_card_image
    profile_photo = self.profile_photo
    profile_photo ? profile_photo.photo.url(size) : self.id_card_image(size)
  end

  before_save :set_preferred_name, :clean_arrays, :parameterize_groupings
  validates :slug, uniqueness: true, allow_nil: true
  validates :first_name, :last_name, presence: true
  validates :gender, inclusion: { in: GENDERS, allow_nil: true }
  validates :faculty_status,
            inclusion: { in: FACULTY_STATUSES, allow_nil: true }

  scope :from_biola, -> { where(:biola_id.ne => nil) }
  scope :not_from_biola, -> { where(biola_id: nil) }
  scope :imported_from_trogdir, -> { where(:trogdir_id.ne => nil) }
  scope :not_imported_from_trogdir, -> { where(trogdir_id: nil) }

  def to_s
    name || ''
  end

  def birthday
    # Should return month and day as string. Example: "February 03"
    birth_date.strftime('%B %d') if birth_date
  end

  def first_initial
    preferred_name.to_s[0] || ''
  end

  def legal_first_initial
    first_name.to_s[0] || ''
  end

  def last_initial
    last_name.to_s[0] || ''
  end

  def name
    if display_name.present?
      display_name
    else
      [preferred_name, last_name].join(' ').squeeze(' ')
    end
  end

  def legal_name
    [first_name, last_name].join(' ').squeeze(' ')
  end

  def full_legal_name
    [first_name, middle_name, last_name].join(' ').squeeze(' ')
  end

  def middle_initial
    middle_name[0, 1] if middle_name
  end

  ALL_AFFILIATIONS.each do |affl|
    next if affl == :student

    define_method "#{affl}?" do
      affiliations_to_sym.include? affl
    end
  end

  def accepted_student_only?
    accepted_student? && (affiliations_to_sym - [:accepted_student]).empty?
  end

  def student?
    (affiliations_to_sym & STUDENT_ROLES).any?
  end

  def student_only?
    student? && (affiliations_to_sym - STUDENT_ROLES).empty?
  end

  def alumnus_only?
    alumnus? && (affiliations_to_sym - [:alumnus]).empty?
  end

  def affiliations_to_sym
    @aff_sym ||= affiliations.to_a.map do |a|
      a.to_s.parameterize.underscore.to_sym
    end
  end

  def visible_affiliations
    # list all affiliations minus student or alumnus if they are private
    affiliations_to_remove = privacy? ? %i[student alumnus] : []
    affiliations_to_remove += %i[accepted_student] # any affiliation that should never show up to anyone
    affiliations.to_a.map { |aff| aff.to_s.gsub(' ', '_').to_sym } - affiliations_to_remove
  end

  def as_indexed_json(*)
    {
      slug: slug,
      affiliations: visible_affiliations,
      department: department_title,
      departments: departments.to_a.map(&:title),
      department_aliases: departments.to_a.map(&:aliases).flatten,
      email: preferred_biola_email,
      biola_id: biola_id,
      titles: titles,
      biola_title: biola_title,
      faculty_status: faculty_status,
      first_initial: first_initial,
      first_name: first_name,
      full_legal_name: full_legal_name,
      last_initial: last_initial,
      last_name: last_name,
      last_name_raw: last_name,
      legal_name: legal_name,
      middle_name: middle_name,
      normalized_data: {
        title: name,
        subtitles: (titles + [department_title]).compact.presence || [major, minor].compact,
        short_description: nil,
        location: office_location,
        image_url: profile_photo_url(:thumb),
        phone: full_biola_phone_number,
        alternate_phone: full_biola_phone_number(:alternate_employee_phone),
        published: published,
        email: preferred_biola_email
      },
      preferred_name: preferred_name,
      display_name: display_name,
      previous_last_name: previous_last_name,
      major: major,
      minor: minor,
      mailbox: mailbox,
      privacy: privacy,
      is_public: public?,
      show_work_email: show_work_email,
      show_employee_phone: show_employee_phone
    }
  end

  def empty_alumnus?
    visible_affiliations == [:alumnus] &&
      preferred_biola_email.blank? &&
      full_biola_phone_number.blank? &&
      alternate_employee_phone.blank?
  end

  mapping do
    %i[first_name last_name preferred_name].each do |attribute|
      indexes attribute, type: 'string', analyzer: 'edge_ngram_analyzer'
    end
    indexes 'normalized_data.title', type: 'string', index: 'not_analyzed'
  end

  def dont_index?
    private? || accepted_student_only? || !published || empty_alumnus?
  end

  def public?
    return true if faculty?
    return false if student? || private?
    publicly_viewable
  end
  alias is_public? public?

  def private?
    return false if faculty? || employee?
    privacy
  end

  private

  def set_preferred_name
    self.preferred_name = preferred_name.presence || first_name
  end

  def clean_arrays
    %i[affiliations groupings].each do |att|
      next if send(att).nil?
      send("#{att}=", send(att).reject(&:blank?))
    end
  end

  def parameterize_groupings
    return unless groupings.present?
    groupings.map!(&:parameterize)
  end

  def update_bio_slug
    return if bio_editions.empty? || slug == bio_editions.first&.slug
    bio_editions.first.update_attribute(:slug, slug)
  end
end
