class ServiceLink
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include Elasticsearchable
  include StringAccessorForArray

  field :title, type: String
  field :description, type: String
  field :slug, type: String # Maybe useful for page describing service
  field :url, type: String # url to service

  field :topics, type: Array, default: [] # used for organizing services
  field :keywords, type: Array, default: [] # used for searching
  string_accessor_for_array :topics
  string_accessor_for_array :keywords

  embeds_one :audience_collection
  # has_and_belongs_to_many :site_categories  #(support page topics / categories)
  mount_uploader :logo, ProfilePhotoUploader

  validates_presence_of :slug, :title
  validates_format_of :slug, with: /\A[\w\-\/]+\Z/i
  validates_uniqueness_of :slug, :title

  before_validation :set_slug, unless: :slug?

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :custom_search, (lambda do |query|
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, ::Regexp::IGNORECASE)
        any_of({title: q}, {slug: q}, {url: q})
      end
    rescue ::RegexpError
      ServiceLink.none
    end
  end)

  alias :to_s :title

  def as_indexed_json(*)
    {
      title: title,
      slug: slug,
      description: description,
      url: url,
      topics: topics,
      keywords: keywords
    }
  end

  mapping do
    indexes "title", type: 'string', fields: {
      raw:  { type: 'string', index: 'not_analyzed' },
      edge: { type: 'string', analyzer: 'edge_ngram_analyzer' }
    }
  end

  protected
  def set_slug
    self.slug = title.parameterize if title?
  end

end
