require 'uri'

class Site
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include PermissionsSubject
  include ActsAsWebPage
  include HasPresentationData

  SITE_ROLES = { "Site" => [["Admin", :site_admin]] }
  ARTICLE_ROLES = { "Article" => [["Publisher", :article_publisher], ["Editor", :article_editor], ["Author", :article_author]] }
  EVENT_ROLES = { "Event" =>[["Publisher", :event_publisher], ["Editor", :event_editor], ["Author", :event_author]] }
  FEATURE_ROLES = { "Feature" =>[["Publisher", :feature_publisher], ["Editor", :feature_editor], ["Author", :feature_author]] }
  PAGE_EDITION_ROLES = { "Page Edition" =>[["Publisher", :page_edition_publisher], ["Editor", :page_edition_editor], ["Author", :page_edition_author]] }

  FULL_ROLES = ["site_admin", "article_publisher", "article_editor", "article_author",
                "event_publisher", "event_editor", "event_author",
                "feature_publisher", "feature_editor", "feature_author",
                "article_admin", "event_admin", "feature_admin"]

  has_and_belongs_to_many :features
  has_many :site_categories
  has_many :articles
  has_many :events
  has_many :feature_locations
  has_many :page_editions
  has_many :menus

  field :title, type: String
  field :url, type: String
  field :has_articles, type: Boolean, default: false
  field :has_events, type: Boolean, default: false
  field :has_features, type: Boolean, default: false
  field :has_audience_collections, type: Boolean, default: false
  field :has_page_editions, type: Boolean, default: false
  field :slug, type: String
  field :staging_url, type: String
  field :article_author_roles, type: Array
  field :event_author_roles, type: Array
  field :site_layout, type: String

  before_validation :set_slug, unless: :slug?

  validates :slug, presence: true, uniqueness: true
  validates_format_of :slug, with: /\A[\w\-]+\Z/i
  validates_presence_of :title, :url
  validates :url, absolute_uri: true
  validates_uniqueness_of :title

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :with_articles_enabled, -> { where has_articles: true }
  scope :with_events_enabled, -> { where has_events: true }
  scope :with_features_enabled, -> { where has_features: true }
  scope :with_page_editions_enabled, -> { where has_page_editions: true }

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q})
      end
    rescue ::RegexpError
      Site.none
    end
  }

  alias :to_s :title

  def categories(type)
    site_categories.where type: type
  end

  def enabled_roles
    roles = SITE_ROLES
    roles = roles.merge(ARTICLE_ROLES) if has_articles
    roles = roles.merge(EVENT_ROLES) if has_events
    roles = roles.merge(FEATURE_ROLES) if has_features
    roles = roles.merge(PAGE_EDITION_ROLES) if has_page_editions
    return roles
  end

  def enabled_mediums
    mediums = []
    mediums << "Article" if has_articles
    mediums << "Event" if has_events
    return mediums
  end

  protected
  def set_slug
    self.slug = title.parameterize if title?
  end
end
