class SiteCategory
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable
  include PermissionsSubject
  include Elasticsearchable

  CATEGORY_TYPES = ['Event', 'Article', 'Page Edition']

  belongs_to :site
  # has_and_belongs_to_many does not support polymorphism so we need to define each separately
  has_and_belongs_to_many :articles
  has_and_belongs_to_many :events
  has_and_belongs_to_many :page_editions

  field :title, type: String
  field :type, type: String
  field :slug, type: String
  field :description, type: String

  before_validation :set_slug, unless: :slug?

  validates_uniqueness_of :slug, scope: [:site_id, :type]
  validates_format_of :slug, with: /\A[\w\-]+\Z/i
  validates_presence_of :title, :type, :slug
  validates_uniqueness_of :title, scope: [:site_id, :type]

  track_history on: :fields, track_create: true, track_update: true, track_destroy: true

  scope :custom_search, -> query {
    begin
      if query.present?
        q = ::Regexp.new(query.to_s, 'i')
        any_of({title: q})
      end
    rescue ::RegexpError
      SiteCategory.none
    end
  }

  alias :to_s :title

  def as_indexed_json(*)
    {
      title: title,
      slug: slug,
      category_type: type,
      site_id: site_id.to_s,
      description: description
    }
  end

  mapping do
    indexes "title", type: 'string', fields: {
      raw:  { type: 'string', index: 'not_analyzed' },
      edge: { type: 'string', analyzer: 'edge_ngram_analyzer' }
    }
  end

  protected
  def set_slug
    self.slug = title.parameterize if title?
  end
end
