class User
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::History::Trackable

  include PermissionsActor

  after_save :create_or_update_person!
  before_save :downcase_username

  belongs_to :person

  field :trogdir_id, type: String
  field :username, type: String
  field :first_name, type: String
  field :last_name, type: String
  field :biola_id, type: Integer
  field :title, type: String
  field :email, type: String
  field :photo_url, type: String
  # TODO: do we really need this in addition to
  # Person#department and Person#departments?
  field :department, type: String
  field :affiliations, type: Array, default: []
  field :entitlements, type: Array, default: []
  alias_attribute :netid, :username

  # for logging in
  field :current_login_at, type: DateTime
  field :last_login_at, type: DateTime
  field :login_count, type: Integer

  has_many :articles
  has_many :events
  validates :username, presence: true, uniqueness: true
  validates :trogdir_id, presence: true, uniqueness: true

  track_history on: :fields,
                track_create: true,
                track_update: true,
                track_destroy: true

  scope :custom_search, (lambda do |query|
    if query.present?
      nested_conditions = []
      # We need to take into account first and last name, which is why we
      # split up the query and link all the words together with an '$and'
      # filter so all the words need to match something.
      query.split.each do |q|
        fields = %i[first_name last_name]
        conditions = fields.map do |f|
          { f => ::Regexp.new("^#{q}", ::Regexp::IGNORECASE) }
        end
        nested_conditions << { '$or' => conditions }
      end
      where('$and' => nested_conditions)
    end
  end)

  Person::ALL_AFFILIATIONS.each do |affl|
    define_method "#{affl}?" do
      role? affl
    end
  end

  # Student worker has a space in it so it needs some special attention
  def student_worker?
    has_role?(:"student worker") || has_role?(:student_worker)
  end

  # Person should always have a trogdir id associated with it from the related
  # user record
  def find_person
    @user_person ||= Person.where(trogdir_id: trogdir_id).first
  end

  def tracking_id
    # Warning: Changing this will cause duplicate records in Intercom
    Digest::MD5.hexdigest(biola_id.to_s)
  end

  def has_published_bio?
    BioEdition.published.where(person_id: person.id).count > 1
  end

  # POLICY: All CAS users signing into wcms should have trogdir_ids. Which
  # is to say that they have a trogdir account. Elsewise they should get 403
  def create_or_update_person!
    return unless trogdir_id.present?

    person = find_person
    if person
      person.update_attributes(update_person_attributes)
    else
      person = Person.create(new_person_attributes)
    end

    return if self.person.present? || !person.persisted?

    self.person = person
    save
  end

  def grant_auth_over_own_person
    # TODO: Should we remove this method?
    # => The PersonPolicy is going to always return true so this method will
    # never create a permission entry.
    # => It already allows a user to edit their own person record without need
    # a permission entry
    person.authorize! self, :edit unless person.nil? || policy(person).edit?
  end

  def name
    [first_name, last_name].compact.join(' ')
  end

  def to_s
    name
  end

  def update_login_info!
    self.last_login_at = current_login_at
    self.current_login_at = Time.now
    self.login_count = login_count.to_i + 1
    save
  end

  def role?(*roles)
    (self.roles & roles).any?
  end
  alias has_role? role?

  attr_writer :roles

  def roles
    @roles ||=
      (Array(affiliations) + relevant_entitlements).reject(&:blank?).map do |i|
        i.parameterize.underscore.to_sym
      end
  end

  def admin?
    has_role?(:admin) || developer?
  end

  def developer?
    has_role?(:developer)
  end

  def self.role_to_entitlement(role)
    BuwebContentModels.config.urn_namespaces.map do |namespace|
      "#{namespace}#{role}"
    end
  end

  def restricted_roles
    return [] if (roles & %i[developer employee faculty]).any?
    if (roles & %i[accepted_student student student_worker]).any?
      return [:alumnus]
    end
    return %i[alumnus accepted_student student student_worker] if friend?

    %i[accepted_student student student_worker]
  end

  def visible_roles
    return [:faculty] if roles.blank?
    Person::ALL_AFFILIATIONS - restricted_roles
  end

  private

  def downcase_username
    username&.downcase!
  end

  # Find URNs that match the namespaces and remove the namespace
  # See http://en.wikipedia.org/wiki/Uniform_Resource_Name
  def relevant_entitlements
    urns = Array(entitlements)
    nids = BuwebContentModels.config.urn_namespaces

    return [] if urns.blank?

    clean_urns = urns.map { |e| e.gsub(/^urn:/i, '') }
    clean_nids = nids.map { |n| n.gsub(/^urn:/i, '') }

    clean_urns.map do |urn|
      clean_nids.map do |nid|
        urn[0...nid.length] == nid ? urn[nid.length..urn.length] : nil
      end
    end.flatten.compact
  end

  def new_person_attributes
    { trogdir_id: trogdir_id,
      biola_id: biola_id,
      biola_photo_url: photo_url,
      biola_email: email,
      department: department,
      biola_title: title,
      first_name: first_name,
      preferred_name: nil,
      last_name: last_name,
      affiliations: affiliations }
  end

  def update_person_attributes
    {
      biola_photo_url: photo_url,
      department: department,
      affiliations: affiliations
    }
  end
end
