# NOTE: `presentation_data` gets overwritten every time presentation_yaml is saved.
#
# SECURITY NOTE: Use YAML.safe_load() instead of YAML.load(). This will prevent any
#                unwanted security vulnerabilities.
#                http://www.sitepoint.com/anatomy-of-an-exploit-an-in-depth-look-at-the-rails-yaml-vulnerability/
#                http://www.ruby-doc.org/stdlib-2.1.2/libdoc/psych/rdoc/Psych.html#method-c-safe_load
module HasPresentationData
  extend ActiveSupport::Concern
  include HasJsonSchema

  included do

    # This is the field you should read from on the front end sites.
    field :presentation_data, type: Hash, default: {}

    # The schema will get coppied from the selected template
    field :presentation_data_schema, type: Array, default: []
    # TODO: Remove presentation_data_schema after migrating data to presentation_data_json_schema
    field :presentation_data_json_schema, type: String
    create_json_schema_validations :presentation_data_json_schema
    validate :validate_json_schema # this method is defined in HasJsonSchema

    # Specifies whether we should keep the schema in sync with the template
    field :keep_in_sync, type: Boolean, default: true

    # Gets schema from the selected template
    #   I would make this a polymorphic relationship, but they don't work
    #   in this direction with Mongoid. So right now there is no way to
    #   refer from the template to here, but that is OK I think.
    belongs_to :presentation_data_template

    # Only sync schema before create
    before_save :conditionally_sync_schema
    before_create :sync_schema  # we need to make sure this runs at least once on create

    def sync_schema
      if schema = presentation_data_template.try(:schema)
        self.presentation_data_schema = schema
      end
      if json_schema = presentation_data_template.try(:json_schema)
        self.presentation_data_json_schema = json_schema
      end
    end

    def conditionally_sync_schema
      sync_schema if keep_in_sync
    end

    def presentation_data=(hash)
      if hash.is_a? Hash
        # TODO: convert hash arrays into real arrays before saving.
        write_attribute :presentation_data, hash
      else
        # TODO: Maybe we should just throw an error here.
        write_attribute :presentation_data, {}
      end
    end

    def presentation_data_json
      JSON.pretty_generate(presentation_data)
    end

    def presentation_data_json=(json)
      begin
        self.presentation_data = JSON.parse(json)
      rescue JSON::ParserError
        return false
      end
    end


    ########################
    # Depricating the following fields and methods.
    # I don't want to use presentation_yaml anymore.
    # Now we are generating a form for people to edit,
    #   which gets saved directly as a hash.
    # If we want to edit the raw data in the future,
    #   it will have to be as JSON istead of YAML
    #######################

    # Write to this attribute from the publisher.
    #  When you save valid YAML, it saves a hashed copy into `presentation_data`
    field :presentation_yaml, type: String

    validate :is_valid_yaml?

    def presentation_yaml=(string)
      begin
        write_attribute :presentation_yaml, string
        self.presentation_data = YAML.safe_load(string) # this will throw an error if YAML is invalid
      rescue
        # setting error here won't do anything unfortunately... So we need a validation method.
      end
    end

    def is_valid_yaml?
      begin
        # only validate if presentation_yaml has content and that content has changed
        if presentation_yaml.present? && presentation_yaml_changed?
          data = YAML.safe_load(presentation_yaml)  # this will throw an error if YAML is invalid
          raise 'invalid YAML' if data.present? && !data.is_a?(Hash)
        end
      rescue
        self.errors.add(:presentation_yaml, "is not valid YAML")
      end
    end
  end
end
